"""
    Test script to be loaded with pywbemcli --mock-server general option to test
    capability to execute python code as part of startup.
    This script loads a single class into the repository.
    This script includes  asserts to confirm that the class is loaded into
    the repositoryand to verify that the GLOBALS are passed to this code from
    pywbemcli.
"""
from pywbem import CIMQualifier, CIMClass, CIMProperty, CIMMethod

# test that GLOBALS exist
assert "CONN" in globals()
assert 'SERVER' in globals()
assert 'VERBOSE'in globals()


def build_classes():
    """
    Function that builds and returns a single class: CIM_Foo that will to be
     used as a test class for the mock class tests.
    """
    # build the key properties
    qkey = {'Key': CIMQualifier('Key', True)}
    dkey = {'Description': CIMQualifier('Description', 'blah blah')}

    # build the CIMClass with properties and methods.
    c = CIMClass(
        'CIM_FooDirLoad', qualifiers=dkey,
        properties={'InstanceID':
                    CIMProperty('InstanceID', None, qualifiers=qkey,
                                type='string', class_origin='CIM_Foo',
                                propagated=False)},
        methods={'Delete': CIMMethod('Delete', 'uint32', qualifiers=dkey,
                                     class_origin='CIM_Foo',
                                     propagated=False),
                 'Fuzzy': CIMMethod('Fuzzy', 'string', qualifiers=dkey,
                                    class_origin='CIM_Foo',
                                    propagated=False)})
    # add the objects to the mock repository
    CONN.add_cimobjects(c)  # noqa: F821


build_classes()
assert(CONN.GetClass('CIM_FooDirLoad'))  # noqa: F821



"""
The class in this file defines a common base for the pytests executed
through pywbemcli execution

"""

from __future__ import absolute_import, print_function
import re
import os
import pytest
import six

from .utils import execute_pywbemcli, assert_rc, assert_patterns, assert_lines

TEST_DIR = os.path.dirname(__file__)


class CLITestsBase(object):
    # pylint: disable=too-few-public-methods, useless-object-inheritance
    """
        Defines methods to execute tests on pywbemcli.

    """
    def command_test(self, desc, command_grp, inputs, exp_response, mock_files,
                     condition, verbose=None):
        # pylint: disable=line-too-long, no-self-use
        """
        Test method to execute test on pywbemcli by calling the executable
        pywbemcli for the command defined by command with arguments defined
        by args. This can execute pywbemcli either with a mock environment
        by using the mock_files variable or without mock if the mock_files
        parameter is None. The method tests the results of the execution
        of pywbemcli using the exp_response parameter.

        Parameters:
          desc (:term:`string`):
            Description of the test

          command_grp (:term:`string`):
            pywbemcli command_grp definedfor this test.  This is the first
            level of the command(the command group) (ex. class).

          inputs (:class:`py:dict` or :class:`py:list` of :term:`string` or :term:`string`):

            If inputs is a list of strings, it contains the command
            line arguments, without the command name.

            Each single argument must be its own item in the iterable; combining
            the arguments into a string does not work.
            The arguments may be binary strings encoded in UTF-8, or unicode
            strings.

            If inputs is a dict it can contain the following possible keys:

              * 'args': defines local arguments to append to the command after
                  the command name. Each single argument must be its own
                  item in the iterable.

              * 'general': defines general arguments that will be inserted
                  into the command line before the command name. Each
                  single argument must be its own item in the iterable.

              * 'env': Dictionary of environment variables where  key is the
                variable name as a :term:`string`; dict value is the variable
                value as a :term:`string` (without any shell escaping needed).
                If None, no environment variables are set for the test.

              * 'stdin': If the key is `stdin`, the value is either a string or
                list of strings. If `stdin` is a list or tuple of strings, each
                string defines a line of text that will be included as stdin to
                the call to pywbemcli. The iterable is mapped to a single string
                with each line separated by an EOL char. This becomes the stdin
                parameter for the call to pywbemcli. Each line is processed
                as a separate repl line by the pybemcli executable.
                If `stdin` is a single string it is passed  to the stdin
                parameter for the call to pywbemcli. If `stdin` is defined, the
                parameters defining a command to be executed are ignored and
                the full command must be in the `stdin` iterable.

                If None, no stdin input is defined for the test.

          exp_response (:class:`py:dict`)

            Keyword arguments for expected response.

            Includes the following possible keys:

               'stdout' or 'stderr' - Defines which return is expected (the
               expected response). The value is a string or iterable defining
               the data expected in the response. The data definition in
               this dictionary entry must be compatible with the definition
               of expected data for the test selected.
               Only one of these keys may exist.

               'test' - If it exists defines the test used to compare the
               returned data with the expected returned data defined as the
               value of 'stdout'/'stderr'.

               The tests define are:

                 'startswith' - Expected Response must be a single string
                 The returned text defined starts with the defined string

                 'lines' -  Expected response may be either a list of strings or
                 single string

                 Compares for exact match between the expected response and the
                 returned data line by line. The number of lines and the data
                 in each line must match.  If the expected response is a single
                 string it is split into lines separated at each new line
                 before the match

                 'linesnows' - Expected response may be either list of strings
                 or single string. Compares as with lines except that all
                 whitespace is removed from the strings before the compare.

                 'patterns' - Expected response must be same as lines test
                 except that each line in the expected response is treated as
                 a regex expression and a regex match is executed for each line.

                 'regex' - Expected response is a single string or list of
                 strings. Each string is considered a regex expression. The
                 regex expression is tested against each line in the output
                 using regex search. All strings in the expected response must
                 be found in actual response to pass the test

                 'in' - Expected response is string or list of strings.
                 Tests the complete response line by line to determine if each
                 entry in expected response is in the response data as a single
                 test.

                 'innows' - Like 'in, except that differences in whitespace are
                 ignored.

                 Executes a single regex search of the entire
                 response data to match with each entry in the expected
                 response

               'rc' expected exit_code from pywbemcli.  If None, code 0
               is expected.

          mock_files (:term:`string` or list of string or None):
            If this is a string, this test will be executed using the
            --mock-server pywbemcl option with this file as the name of the
            objects to be compiled or executed. This should be just a file name
            and this method assumes the file is in the tests/unit directory.

            If it is a list, the same rules apply to each entry in the list.

            If None, test is executed without the --mock-server input parameter
            and defines an artificial server name  Used to test commands
            and options that do not communicate with a server.  It is faster
            than installing the mock repository

          condition (None or False):
            If False, the test is skipped

          verbose (:class:`py:bool`):
            If `True` the assembled command line will be displayed
        """  # noqa: E501
        # pylint: enable=line-too-long

        if not condition:
            pytest.skip("Condition for test case %s not met" % desc)

        env = None
        stdin = None
        if isinstance(inputs, dict):
            general_args = inputs.get("general", None)
            local_args = inputs.get("args", None)
            env = inputs.get("env", None)
            stdin = inputs.get('stdin', None)
            if stdin:
                if isinstance(stdin, (tuple, list)):
                    stdin = '\n'.join(stdin)
        elif isinstance(inputs, six.string_types):
            local_args = inputs.split(" ")
            general_args = None
        elif isinstance(inputs, (list, tuple)):
            local_args = inputs
            general_args = None
        else:
            assert 'Invalid inputs param to test %r . Allowed types are ' \
                   'dict, string, list, tuple.' % inputs

        if isinstance(local_args, six.string_types):
            local_args = local_args.split(" ")

        if isinstance(general_args, six.string_types):
            general_args = general_args.split(" ")

        cmd_line = []
        if general_args:
            cmd_line.extend(general_args)

        if mock_files:
            if isinstance(mock_files, (list, tuple)):
                for item in mock_files:
                    cmd_line.extend(['--mock-server',
                                     os.path.join(TEST_DIR, item)])
            elif isinstance(mock_files, six.string_types):
                cmd_line.extend(['--mock-server',
                                 os.path.join(TEST_DIR, mock_files)])
            else:
                assert("CLI_TEST_EXTENSIONS mock_file %s invalid" % mock_files)

        if not stdin:
            cmd_line.append(command_grp)

        if local_args:
            cmd_line.extend(local_args)

        if verbose:
            print('\nCMDLINE: %s' % cmd_line)

        if verbose and env:
            print('ENV: %s' % env)

        rc, stdout, stderr = execute_pywbemcli(cmd_line, env=env, stdin=stdin,
                                               verbose=verbose)

        exp_rc = exp_response['rc'] if 'rc' in exp_response else 0
        assert_rc(exp_rc, rc, stdout, stderr)

        if verbose:
            print('RC=%s\nSTDOUT=%s\nSTDERR=%s' % (rc, stdout, stderr))

        if exp_response['test']:
            test_definition = exp_response['test']
        else:
            test_definition = None

        if 'stdout' in exp_response:
            test_value = exp_response['stdout']
            rtn_value = stdout
            rtn_type = 'stdout'
        elif 'stderr' in exp_response:
            test_value = exp_response['stderr']
            rtn_value = stderr
            rtn_type = 'stderr'
        else:
            assert False, 'Expected "stdout" or "stderr" key. One of these ' \
                          'keys required in exp_response.'

        if test_definition:
            if 'test' in exp_response:
                test_definition = exp_response['test']
                # test that rtn_value starts with test_value
                if test_definition == 'startswith':
                    assert isinstance(test_value, six.string_types)
                    assert rtn_value.startswith(test_value), \
                        "{}\n{}={!r}".format(desc, rtn_type, rtn_value)
                # test that lines match between test_value and rtn_value
                # base on regex match
                elif test_definition == 'patterns':
                    if isinstance(test_value, six.string_types):
                        test_value = test_value.splitlines()
                    assert isinstance(test_value, (list, tuple))
                    assert_patterns(test_value, rtn_value.splitlines(),
                                    "{}\n{}={!r}".format(desc, rtn_type,
                                                         rtn_value))
                # test that each line in the test value matches the
                # corresponding line in the rtn_value exactly
                elif test_definition == 'lines':
                    if isinstance(test_value, six.string_types):
                        test_value = test_value.splitlines()
                    if isinstance(test_value, (list, tuple)):
                        assert_lines(test_value, rtn_value.splitlines(),
                                     "{}\n{}={!r}".format(desc, rtn_type,
                                                          rtn_value))
                    else:
                        assert(isinstance(test_value, six.string_types))
                        assert_lines(test_value.splitlines(),
                                     rtn_value.splitlines(),
                                     "{}\n{}={!r}".format(desc, rtn_type,
                                                          rtn_value))
                # compress test_value and rtn_value into whitespace single
                # strings and assert_lines.
                elif test_definition == 'linesnows':
                    assert_lines([remove_ws(test_value)],
                                 [remove_ws(rtn_value)],
                                 "{}\n{}={!r}".format(desc, rtn_type,
                                                      rtn_value))

                # test with a regex search that all values in list exist in
                # the return. Build rtn_value into single string and do
                # re.search against it for each test_value
                elif test_definition == 'regex':
                    assert(isinstance(rtn_value, six.string_types))

                    for regex in test_value:
                        assert isinstance(regex, six.string_types)
                        match_result = re.search(regex, rtn_value, re.MULTILINE)
                        assert match_result, \
                            "Desc: {}\nRegex: {}\nNot in\n{!r}".format(
                                desc, regex, rtn_value)
                elif test_definition == 'in':
                    if isinstance(test_value, six.string_types):
                        test_value = [test_value]
                    for test_str in test_value:
                        assert test_str in rtn_value, \
                            "Desc: {}\nString: {}\nNot in:\n{!r}".format(
                                desc, test_str, rtn_value)
                elif test_definition == 'innows':
                    if isinstance(test_value, six.string_types):
                        test_value = [test_value]
                    for test_str in test_value:
                        assert remove_ws(test_str) in remove_ws(rtn_value), \
                            "Desc: {}\nString: {}\nNot in (ws ignored):\n" \
                            "{!r}".format(desc, test_str, rtn_value)
                elif test_definition == 'not-innows':
                    if isinstance(test_value, six.string_types):
                        test_value = [test_value]
                    for test_str in test_value:
                        assert remove_ws(test_str) not in \
                            remove_ws(rtn_value), \
                            "Desc: {}\nString: {}\nNot in (ws ignored):\n" \
                            "{!r}".format(desc, test_str, rtn_value)
                else:
                    assert 'test %s is invalid. Skipped' % test_definition


def remove_ws(inputs):
    """
    Remove all whitespace from a tuple or list of strings or a single
    string and return the result as a single string. Whitespace is
    defined as spaces, tabs, newlines
    """
    if isinstance(inputs, (tuple, list)):
        inputs = "".join(inputs)

    # return inputs.replace(" ", "").replace("\t", "").replace("\n", "")
    return re.sub(r"\s", "", inputs)



# (C) Copyright 2017 IBM Corp.
# (C) Copyright 2017 Inova Development Inc.
# All Rights Reserved
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
Functions to persist and restore the connections table.  This works from
a file that maintains defined connections. If any exist they are
loaded at startup and available through an interactive session.  Functions
are provided to create, delete, and view existing connections. A set function
allows setting the current active connection into the repository.
"""
from __future__ import absolute_import

import os
import yaml
import six
from ._pywbem_server import PywbemServer

if six.PY2:
    import codecs  # pylint: disable=wrong-import-order


DEFAULT_CONNECTIONS_FILE = 'pywbemcli_connection_definitions.yaml'

DEFAULT_CONNECTIONS_PATH = os.path.join(os.getcwd(), DEFAULT_CONNECTIONS_FILE)


class ConnectionRepository(object):
    # pylint: disable=useless-object-inheritance
    """
    Manage the set of connections defined.  The data for the predefined
    connection exists on disk between pywbemcli sessions and within an
    instance of ConnectionRepository while pywbemcli is running.
    """
    # class variables
    _pywbemcli_servers = {}
    _loaded = False
    _connections_file = None
    connections_group_name = 'connection_definitions'
    default_connection_grp_name = 'default_connection_name'

    # default connection name Must be the name of a
    # connection in the connections file or None.
    default_connection_name = None

    # class level variable so
    def __init__(self, connections_file=None):
        """
        Initialize the object instance if it has not already been initialized
        (class level variable is not None)by reading the connection file.
        """
        if not ConnectionRepository._loaded:
            if connections_file is None:
                ConnectionRepository._connections_file = \
                    DEFAULT_CONNECTIONS_PATH
            else:
                ConnectionRepository._connections_file = connections_file
            self._read_connections_file()

        else:
            if connections_file is not None and \
                    connections_file != self._connections_file:
                raise ValueError("Cannot change connection file name after"
                                 "initalization original {} new {}".
                                 format(self._connections_file,
                                        connections_file))

    @property
    def connections_file(self):
        """
        Return the current connections file
        """
        return self._connections_file

    def __repr__(self):
        """
        Return a string representation of the
        servers dictionary that is suitable for debugging.

        The order of items in the result is the preserved order of
        adding or deleting items.

        The lexical case of the keys in the result is the preserved lexical
        case.
        """
        # items = [_format("{0!A}: {1!A}", key, value)
        #         for key, value in self._pywbemcli_servers.iteritems()]
        items = []
        for key, value in self._pywbemcli_servers.items():
            items.append("%s: %s" % (key, value))
        items_str = ', '.join(items)
        return "{0.__class__.__name__}({{{1}}}, default_connection {2})]". \
            format(self, items_str, self.default_connection_name)

    def __contains__(self, key):
        return key in self._pywbemcli_servers

    def __getitem__(self, key):
        return self._pywbemcli_servers[key]

    def __delitem__(self, key):
        del self._pywbemcli_servers[key]
        self._write_file()

    def __len__(self):
        return len(ConnectionRepository._pywbemcli_servers)

    def __iter__(self):
        return six.iterkeys(ConnectionRepository._pywbemcli_servers)

    def items(self):
        """
        Return a list of the items in the server repo
        """
        return list(self.__iteritems__())

    def keys(self):
        """
        Return a copied list of the dictionary keys, in their original case.
        """
        return list(self.iterkeys())

    def __iteritems__(self):  # pylint: disable=no-self-use
        return six.iteritems(self._pywbemcli_servers)

    def iterkeys(self):
        """
        Return an iterator through the dictionary keys in their original
        case, preserving the original order of items.
        """
        for item in six.iterkeys(self._pywbemcli_servers):
            yield item

    def iteritems(self):
        """
        Return an iterator through the dictionary items, where each item is a
        tuple of its original key and its value, preserving the original order
        of items.
        """
        for item in six.iteritems(self._pywbemcli_servers):
            yield item[1]

    def _read_connections_file(self):
        """
        If there is a file, read it in and install into the dictionary.

        """
        if os.path.isfile(self._connections_file):
            with self.open_file(self._connections_file, 'r') as _fp:
                try:
                    dict_ = yaml.safe_load(_fp)
                    # put all the connection definitions into a group
                    # in the connection file
                    connections_dict = dict_[
                        ConnectionRepository.connections_group_name]

                    ConnectionRepository.default_connection_name = dict_[
                        ConnectionRepository.default_connection_grp_name]
                    try:
                        for name, svr in six.iteritems(connections_dict):
                            ConnectionRepository._pywbemcli_servers[name] = \
                                PywbemServer.create(
                                    replace_underscores=True, **svr)
                            ConnectionRepository._loaded = True
                    except KeyError as ke:
                        raise KeyError("Items missing from record %s in "
                                       "connections file %s" %
                                       (ke, self._connections_file))
                except ValueError as ve:
                    raise ValueError("Invalid YAML in connections file %s. "
                                     "Exception %s" %
                                     (self._connections_file, ve))

    def add(self, name, svr_definition):
        """
        Add a new connection to the connections repository or replace an
        existing connection.  Users of this method should check before add if
        they do not want to replace an existing entry.
        """
        assert svr_definition.mock_server is not None  # must be empty list
        ConnectionRepository._pywbemcli_servers[name] = svr_definition
        self._write_file()

    def delete(self, name):  # pylint: disable=no-self-use
        """Delete a definition from the connections repository"""
        del ConnectionRepository._pywbemcli_servers[name]
        # remove default_name if it is the one being deleted
        if name == self.default_connection_name:
            self.default_connection_name = None
        self._write_file()

    @staticmethod
    def open_file(filename, file_mode='w'):
        """
        A static convenience function that performs the open of the connection
        definitions file correctly for different versions of Python.

        This covers the issue where the file should be opened in text mode but
        that is done differently in Python 2 and Python 3.

        The returned file-like object must be closed by the caller.

        Parameters:

          filename(:term:`string`):
            Name of the file where the recorder output will be written

          file_mode(:term:`string`):
            Optional file mode.  The default is 'w' which overwrites any
            existing file.  if 'a' is used, the data is appended to any
            existing file.

        Returns:

          File-like object.
        """
        if six.PY2:
            # Open with codecs to define text mode
            return codecs.open(filename, mode=file_mode, encoding='utf-8')

        return open(filename, file_mode, encoding='utf8')

    def _write_file(self):  # pylint: disable=no-self-use
        """
        Write the connections file if one has been loaded.
        If the dictionary is empty, it attempts to delete the file.

        If there is an existing file it is moved to filename.bak and a new
        current file written.
        """
        conn_dict = {}
        if self._pywbemcli_servers:
            if ConnectionRepository._pywbemcli_servers:
                conn_dict = \
                    {name: value.to_dict() for name, value in
                     ConnectionRepository._pywbemcli_servers.items()}

            # build dictionary for yaml output
            yaml_dict = {ConnectionRepository.connections_group_name: conn_dict,
                         ConnectionRepository.default_connection_grp_name:
                             self.default_connection_name}

            # Write to tmpfile and if successful create backup file and
            # move the tmpfile to be the new connections file contents.
            tmpfile = "%s.tmp" % self._connections_file

            with self.open_file(tmpfile, 'w') as _fp:
                data = yaml.safe_dump(yaml_dict,
                                      encoding=None,
                                      allow_unicode=True,
                                      default_flow_style=False,
                                      indent=4)
                data = data.replace('\n\n', '\n')  # YAML dump dups newlines
                _fp.write(data)
                _fp.flush()

        # create bak file and then rename tmp file
        if os.path.isfile(self._connections_file):
            bakfile = "%s.bak" % self._connections_file
            if os.path.isfile(bakfile):
                os.remove(bakfile)
            if os.path.isfile(self._connections_file):
                os.rename(self._connections_file, bakfile)

        if self._pywbemcli_servers:
            os.rename(tmpfile, self._connections_file)

    def set_default_connection(self, connection_name):
        """
        Set the connection defined by connection_name to be the current
        connection in the connections file.

        This is accomplished by modifying the "current_connection" entry
        and rewriting the file.
        """
        if connection_name in self._pywbemcli_servers:
            ConnectionRepository.default_connection_name = connection_name
            self._write_file()

        else:
            raise ValueError('Connection name "%s" does not exist in set of '
                             'connections' % connection_name)

    def get_default_connection_name(self):
        """
        Returns the name of the current connection in the connections file.
        This may be the name of a connection in the connections file or
        None if no connection is defined as the current connection.
        """
        return self.default_connection_name



# Copyright 2018 IBM Corp. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Test the connection command group and its commands. This test manipulates
a connections file in the test directory.  It assumes that the
test directory is the same as the current working directory where pywbemcli
was called.  If there is a connections file at the start of the test it is
renamed for the test and restored at the end of the test.
"""

import os
import pytest

from .cli_test_extensions import CLITestsBase
from .common_options_help_lines import CMD_OPTION_HELP_HELP_LINE

SCRIPT_DIR = os.path.dirname(__file__)

# A mof file that defines basic qualifier decls, classes, and instances
# but not tied to the DMTF classes.
SIMPLE_MOCK_FILE = 'simple_mock_model.mof'
INVOKE_METHOD_MOCK_FILE = 'simple_mock_invokemethod.py'
MOCK_PROMPT0_FILE = "mock_prompt_0.py"
MOCK_CONFIRMY_FILE = "mock_confirm_y.py"

TEST_DIR = os.path.dirname(__file__)

# Get the relative path to the current directory.  This presumes that the
# test was run from the pywbemtools directory
TEST_DIR_REL = os.path.relpath(TEST_DIR)
MOCK_FILE_PATH = os.path.join(TEST_DIR_REL, SIMPLE_MOCK_FILE)
CONFIRM_FILE_PATH = os.path.join(TEST_DIR_REL, MOCK_CONFIRMY_FILE)

CONNECTION_HELP_LINES = [
    'Usage: pywbemcli connection [COMMAND-OPTIONS] COMMAND [ARGS]...',
    'Command group for WBEM connection definitions.',
    CMD_OPTION_HELP_HELP_LINE,
    'delete  Delete a WBEM connection definition.',
    'export  Export the current connection.',
    'list    List the WBEM connection definitions.',
    'save    Save the current connection to a new WBEM connection definition.',
    'select  Select a WBEM connection definition as current or default.',
    'show    Show a WBEM connection definition or the current connection.',
    'test    Test the current connection with a predefined WBEM request.',
]

CONNECTION_DELETE_HELP_LINES = [
    'Usage: pywbemcli connection delete [COMMAND-OPTIONS] NAME',
    'Delete a WBEM connection definition.',
    CMD_OPTION_HELP_HELP_LINE,
]

CONNECTION_EXPORT_HELP_LINES = [
    'Usage: pywbemcli connection export [COMMAND-OPTIONS]',
    'Export the current connection.',
    CMD_OPTION_HELP_HELP_LINE,
]

CONNECTION_LIST_HELP_LINES = [
    'Usage: pywbemcli connection list [COMMAND-OPTIONS]',
    'List the WBEM connection definitions.',
    CMD_OPTION_HELP_HELP_LINE
]

CONNECTION_SAVE_HELP_LINES = [
    'Usage: pywbemcli connection save [COMMAND-OPTIONS] NAME',
    'Save the current connection to a new WBEM connection definition.',
    CMD_OPTION_HELP_HELP_LINE
]

CONNECTION_SELECT_HELP_LINES = [
    'Usage: pywbemcli connection select [COMMAND-OPTIONS] NAME',
    'Select a WBEM connection definition as current or default.',
    '-d, --default  If set, the connection is set to be the default ',

    CMD_OPTION_HELP_HELP_LINE
]

CONNECTION_SHOW_HELP_LINES = [
    'Usage: pywbemcli connection show [COMMAND-OPTIONS] NAME',
    'Show a WBEM connection definition or the current connection.',
    '--show-password  If set, show existing password in results.',
    CMD_OPTION_HELP_HELP_LINE
]

CONNECTION_TEST_HELP_LINES = [
    'Usage: pywbemcli connection test [COMMAND-OPTIONS]',
    'Test the current connection with a predefined WBEM request.',
    CMD_OPTION_HELP_HELP_LINE,
]

OK = True     # mark tests OK when they execute correctly
RUN = True    # Mark OK = False and current test case being created RUN
FAIL = False  # Any test currently FAILING or not tested yet


TEST_CASES = [
    # desc - Description of test
    # inputs - String, or list of args or dict of 'env', 'args', 'general',
    #          and 'stdin'. See See CLITestsBase.command_test()  for
    #          detailed documentation
    # exp_response - Dictionary of expected responses (stdout, stderr, rc) and
    #                test definition (test: <testname>).
    #                See CLITestsBase.command_test() for detailed documentation.
    # mock - None or name of files (mof or .py),
    # condition - If True, the test is executed,  Otherwise it is skipped.

    ['Verify connection command --help response',
     '--help',
     {'stdout': CONNECTION_HELP_LINES,
      'test': 'innows'},
     None, OK],

    ['Verify connection command -h response',
     '-h',
     {'stdout': CONNECTION_HELP_LINES,
      'test': 'innows'},
     None, OK],

    ['Verify connection command delete --help response',
     ['delete', '--help'],
     {'stdout': CONNECTION_DELETE_HELP_LINES,
      'test': 'innows'},
     None, OK],

    ['Verify connection command delete -h response',
     ['delete', '-h'],
     {'stdout': CONNECTION_DELETE_HELP_LINES,
      'test': 'innows'},
     None, OK],

    ['Verify connection command export --help response',
     ['export', '--help'],
     {'stdout': CONNECTION_EXPORT_HELP_LINES,
      'test': 'innows'},
     None, OK],

    ['Verify connection command export -h response',
     ['export', '-h'],
     {'stdout': CONNECTION_EXPORT_HELP_LINES,
      'test': 'innows'},
     None, OK],

    ['Verify connection command list --help response',
     ['list', '--help'],
     {'stdout': CONNECTION_LIST_HELP_LINES,
      'test': 'innows'},
     None, OK],

    ['Verify connection command list -h response',
     ['list', '-h'],
     {'stdout': CONNECTION_LIST_HELP_LINES,
      'test': 'innows'},
     None, OK],

    ['Verify connection command save --help response',
     ['save', '--help'],
     {'stdout': CONNECTION_SAVE_HELP_LINES,
      'test': 'innows'},
     None, OK],

    ['Verify connection command save -h response',
     ['save', '-h'],
     {'stdout': CONNECTION_SAVE_HELP_LINES,
      'test': 'innows'},
     None, OK],

    ['Verify connection command select  --help response',
     ['select', '--help'],
     {'stdout': CONNECTION_SELECT_HELP_LINES,
      'test': 'innows'},
     None, OK],

    ['Verify connection command select  -h response',
     ['select', '-h'],
     {'stdout': CONNECTION_SELECT_HELP_LINES,
      'test': 'innows'},
     None, OK],

    ['Verify connection command show --help response',
     ['show', '--help'],
     {'stdout': CONNECTION_SHOW_HELP_LINES,
      'test': 'innows'},
     None, OK],

    ['Verify connection command show -h response',
     ['show', '-h'],
     {'stdout': CONNECTION_SHOW_HELP_LINES,
      'test': 'innows'},
     None, OK],

    ['Verify connection command test --help response',
     ['test', '--help'],
     {'stdout': CONNECTION_TEST_HELP_LINES,
      'test': 'innows'},
     None, OK],

    ['Verify connection command test -h response',
     ['test', '-h'],
     {'stdout': CONNECTION_TEST_HELP_LINES,
      'test': 'innows'},
     None, OK],

    ['Verify connection command list empty repository.',
     {'general': ['-o', 'simple'],
      'args': ['list']},
     {'stdout': [
         "WBEM server connections:", ""],
      'test': 'innows'},
     None, OK],

    ['Verify connection command delete, empty repo fails.',
     ['delete', 'blah'],
     {'stderr': ["Connection repository", "empty"],
      'rc': 1,
      'test': 'innows'},
     None, OK],

    ['Verify --mock-server and --server together fail.',
     {'general': ['--mock-server', MOCK_FILE_PATH, '--server', 'http://blah'],
      'args': ['list']},
     {'stderr': ['Conflicting server definitions:',
                 'server:', 'http://blah',
                 'mock-server:', MOCK_FILE_PATH],
      'rc': 1,
      'test': 'innows'},
     None, OK],

    #
    # The following tests are a sequence. Each depends on the previous
    # and what was in the repository when each test is executed.
    #
    ['Verify connection command save creates file.',
     {'general': ['--server', 'http://blah'],
      'args': ['save', 'test1']},
     {'stdout': "",
      'test': 'innows',
      'file': {'before': 'none', 'after': 'exists'}},
     None, OK],

    ['Verify connection command show of just created test1',
     ['show', 'test1'],
     {'stdout': [
         "name: test1", "  server: http://blah",
         "default-namespace: root/cimv2", "  user: None",
         "password: None",
         "timeout: 30",
         "verify: True",
         "certfile: None",
         "keyfile: None", "  mock-server:"],
      'test': 'innows',
      'file': {'before': 'exists', 'after': 'exists'}},
     None, OK],

    ['Verify connection command save with complex general options short',
     {'general': ['--server', 'http://blahblah', '-u', 'fred', '-p',
                  'argh', '-t', '18', '--no-verify', '-l', 'api=file,all'],
      'args': ['save', 'test2']},
     {'stdout': "",
      'test': 'innows',
      'file': {'before': 'exists', 'after': 'exists'}},
     None, OK],

    ['Verify connection command show named connnection test1',
     ['show', 'test1'],
     {'stdout': [
         "name: test1", "  server: http://blah",
         "default-namespace: root/cimv2", "user: None", "password: None",
         "timeout: 30",
         "verify: True",
         "certfile: None",
         "keyfile: None", "mock-server:"],
      'test': 'innows'},
     None, OK],

    ['Verify connection command show  test2 with --show-password',
     ['show', 'test2', '--show-password'],
     {'stdout': ['name: test2',
                 'server: http://blahblah',
                 'default-namespace: root/cimv2',
                 'user: fred',
                 'password: argh',
                 'timeout: 18',
                 'verify: False',
                 'certfile: None',
                 'keyfile: None',
                 'mock-server:'],
      'test': 'innows'},
     None, OK],

    ['Verify connection command show  test2, masked password',
     ['show', 'test2'],
     {'stdout': ['name: test2',
                 'server: http://blahblah',
                 'default-namespace: root/cimv2',
                 'user: fred',
                 'password: ****',
                 'timeout: 18',
                 'verify: False',
                 'certfile: None',
                 'keyfile: None',
                 'mock-server:'],
      'test': 'innows'},
     None, OK],

    ['Verify connection command list with 2 servers defined',
     {'general': ['--output-format', 'plain'],
      'args': ['list']},
     {'stdout': ['WBEM server connections:',
                 'name server namespace user timeout verify',
                 "test1 http://blah root/cimv2 30  True",
                 "test2 http://blahblah  root/cimv2 fred 18  False"],
      'test': 'innows'},
     None, OK],

    ['Verify connection command select test2',
     ['select', 'test2', '--default'],
     {'stdout': ['test2', 'default'],
      'test': 'innows'},
     None, OK],

    ['Verify connection command show test2 includes "current"',
     ['show', 'test2'],
     {'stdout': ['test2', 'http://blahblah', 'current'],
      'test': 'innows'},
     None, OK],

    ['Verify connection command select test2 shows it is current',
     ['select', 'test2'],
     {'stdout': ['test2', 'current'],
      'test': 'innows'},
     None, OK],

    ['Verify connection command select test3 fails',
     ['select', 'test9'],
     {'stderr': ['Connection name "test9" does not exist'],
      'rc': 1,
      'test': 'regex'},
     None, OK],

    ['Verify connection command list with 2 servers defined, not sel after '
     ' next pywbemcli call',
     {'general': ['--output-format', 'plain'],
      'args': ['list']},
     {'stdout': ['WBEM server connections:',
                 '(#: default, *: current)',
                 'name    server namespace user timeout verify log',
                 'test1   http://blah      root/cimv2        30  True',
                 '*#test2   http://blahblah  root/cimv2   fred 18  False'
                 'api=file,all'],
      'test': 'insnows'},
     None, OK],

    ['Verify connection command delete test1',
     ['delete', 'test1'],
     {'stdout': ['Deleted', 'test1'],
      'test': 'innows',
      'file': {'before': 'exists', 'after': 'exists'}},
     None, OK],

    ['Verify connection command test',
     ['test'],
     {'stdout': "Connection successful",
      'test': 'lines'},
     SIMPLE_MOCK_FILE, OK],

    ['Verify connection command delete test2',
     ['delete', 'test2'],
     {'stdout': ['Deleted', 'test2', 'connection'],
      'test': 'innows',
      'file': {'before': 'exists', 'after': 'none'}},
     None, OK],

    ['Verify connection command list empty repository.',
     {'general': ['-o', 'simple'],
      'args': ['list']},
     {'stdout': [
         "WBEM server connections:", ""],
      'test': 'innows',
      'file': {'before': 'none', 'after': 'none'}},

     None, OK],

    #
    # The following tests are a new sequence and depends on empty repo.
    # It creates, shows and deletes a single server definition.
    #
    ['Verify connection command add with all arguments.',
     {'general': ['--server', 'http://blahblah',
                  '--default-namespace', 'root/blahblah',
                  '--user', 'john',
                  '--password', 'pw',
                  '--timeout', '30',
                  '--no-verify',
                  '--certfile', 'mycertfile.pem',
                  '--keyfile', 'mykeyfile.pem', ],
      'args': ['save', 'addallargs']},
     {'stdout': "",
      'test': 'lines',
      'file': {'before': 'none', 'after': 'exists'}},
     None, OK],

    ['Verify connection command show addallargs initial version',
     ['show', 'addallargs'],
     {'stdout': [
         'name: addallargs',
         '  server: http://blahblah',
         '  default-namespace: root/blahblah',
         '  user: john',
         '  password: ******',
         '  certfile: mycertfile.pem',
         '  keyfile: mykeyfile.pem',
         '  mock-server: '],
      'test': 'in'},
     None, OK],

    ['Verify connection command with that overwrites existing name works.',
     {'general': ['--server', 'http://blah',
                  '--default-namespace', 'root/blah',
                  '--user', 'john',
                  '--password', 'pw',
                  '--timeout', '30',
                  '--no-verify',
                  '--certfile', 'mycertfile.pem',
                  '--keyfile', 'mykeyfile.pem', ],
      'args': ['save', 'addallargs']},
     {'stderr': '',
      'rc': 0,
      'test': 'innows',
      'file': {'before': 'exists', 'after': 'exists'}},
     None, OK],

    ['Verify connection command show of name addallargs, overwrite changed',
     ['show', 'addallargs', '--show-password'],
     {'stdout': [
         'name: addallargs',
         '  server: http://blah',
         '  default-namespace: root/blah',
         '  user: john',
         '  password: pw',
         '  certfile: mycertfile.pem',
         '  keyfile: mykeyfile.pem',
         '  mock-server: '],
      'test': 'in'},
     None, OK],

    ['Verify connection command delete addallargs',
     ['delete', 'addallargs'],
     {'stdout': "Deleted",
      'test': 'innows',
      'file': {'before': 'exists', 'after': 'none'}},
     None, OK],

    # uses regex because windows generates set and linux export in statements
    # No file verification required. Does not use file
    ['Verify connection command export current connection',
     {'args': ['export'],
      'general': ['-s', 'http://blah', '-u', 'fred', '-p', 'arghh',
                  '--no-verify',
                  '-c', 'certfile.txt', '-k', 'keyfile.txt', '-t', '12']},
     {'stdout': ['PYWBEMCLI_SERVER=http://blah$',
                 'PYWBEMCLI_DEFAULT_NAMESPACE=root/cimv2$',
                 'PYWBEMCLI_USER=fred$',
                 'PYWBEMCLI_PASSWORD=arghh$',
                 'PYWBEMCLI_TIMEOUT=12$',
                 #   TODO 'PYWBEMCLI_NO_VERIFY=True$',
                 'PYWBEMCLI_CERTFILE=certfile.txt$',
                 'PYWBEMCLI_KEYFILE=keyfile.txt$',
                 # account for windows and non_windows platforms
                 '^(export|set) '],
      'test': 'regex',
      'file': {'before': 'none', 'after': 'none'}},
     None, OK],

    #
    #  Test command line parameter errors, select, show, and delete with
    #  no repository
    #
    ['Verify connection command show with name not in empty repo',
     ['show', 'Blah'],
     {'stderr': ['Name "Blah" not current and no connections file'],
      'rc': 1,
      'test': 'innows',
      'file': {'before': 'none', 'after': 'none'}},
     None, OK],

    ['Verify connection command show with no name empty repo',
     ['show'],
     {'stderr': ['No current connection and no connections file'],
      'rc': 1,
      'test': 'innows'},
     None, OK],

    ['Verify connection command select, empty repo fails',
     ['select'],
     {'stderr': ["Connection repository", "empty"],
      'rc': 1,
      'test': 'innows',
      'file': {'before': 'none', 'after': 'none'}},
     None, OK],

    ['Verify connection command select blah, empty repo fails',
     ['select', 'blah'],
     {'stderr': ["Connection repository", "empty"],
      'rc': 1,
      'test': 'innows',
      'file': {'before': 'none', 'after': 'none'}},
     None, OK],

    ['Verify connection command delete, empty repo fails',
     ['delete'],
     {'stderr': ["Connection repository", "empty"],
      'rc': 1,
      'test': 'innows',
      'file': {'before': 'none', 'after': 'none'}},
     None, OK],


    ['Verify connection command delete, empty repo fails',
     ['delete', 'blah'],
     {'stderr': ["Connection repository", "empty"],
      'rc': 1,
      'test': 'innows',
      'file': {'before': 'none', 'after': 'none'}},
     None, OK],

    #  The following is a sequence that starts with an empty repo
    #  Verify create and save mock connection sequence. Starts with empty
    #  server adds a mock server, shows it, tests if valid, and deletes
    #  the connection
    #

    ['Verify mock connection exists.',
     {'general': ['--mock-server', MOCK_FILE_PATH],
      'args': ['save', 'mocktest']},
     {'stdout': "",
      'test': 'lines',
      'file': {'before': 'none', 'after': 'exists'}},
     None, OK],

    ['Verify connection command shows mock file ',
     ['show', 'mocktest'],
     {'stdout': [
         "name: mocktest",
         "default-namespace: root/cimv2",
         "user: None",
         "password: None",
         "timeout: 30",
         "verify: True",
         "certfile: None",
         "keyfile: None",
         "mock-server:", r"simple_mock_model.mof"],
      'test': 'innows'},
     None, OK],

    ['Verify connection command test against existing mock def',
     {'args': ['test'],
      'general': ['--name', 'mocktest']},
     {'stdout': "Connection successful",
      'test': 'linnows',
      'file': {'before': 'exists', 'after': 'exists'}},
     None, OK],

    ['Verify connection command select mocktest with prompt',
     ['select'],
     {'stdout': "",
      'test': 'in',
      'file': {'before': 'exists', 'after': 'exists'}},
     MOCK_PROMPT0_FILE, OK],


    ['Verify connection command show with prompt',
     ['show', '?'],
     {'stdout': ['name: mocktest'],
      'test': 'innows',
      'file': {'before': 'exists', 'after': 'exists'}},
     MOCK_PROMPT0_FILE, OK],

    ['Verify connection command delete with prompt that selects 0',
     ['delete', ],
     {'stdout': ['Select a connection or Ctrl_C to abort',
                 '0: mocktest',
                 'Input integer between 0 and 0 or Ctrl-C to exit selection:'],
      'test': 'innows',
      'file': {'before': 'exists', 'after': 'none'}},
     MOCK_PROMPT0_FILE, OK],

    ['Verify Add mock server to empty connections file.',
     {'general': ['--mock-server', MOCK_FILE_PATH],
      'args': ['save', 'mocktest']},
     {'stdout': "",
      'test': 'lines',
      'file': {'before': 'none', 'after': 'exists'}},
     None, OK],

    ['Verify connection command delete'
     'verify',
     ['delete', 'mocktest'],
     {'stdout': ['name: mocktest', 'Execute delete'],
      'test': 'onnows',
      'file': {'before': 'exists', 'after': 'none'}},
     MOCK_CONFIRMY_FILE, OK],

    #
    #  Verify Create from mock with cmd line params, save, show, delete works
    #  The following is a sequence of tests that must be run in order
    #  The following test is artifical in that the first create actually
    #  uses the mock but not to really create a server
    #
    ['Verify save server with cmd line params to empty connections file.',
     {'args': ['save', 'svrtest2'],
      'general': ['--server', 'http://blah',
                  '--timeout', '45',
                  '--use-pull', 'no',
                  '--default-namespace', 'root/blah',
                  '--user', 'john',
                  '--password', 'pw',
                  '--verify',
                  '--certfile', 'mycertfile.pem',
                  '--keyfile', 'mykeyfile.pem']},
     {'stdout': "",
      'test': 'innows',
      'file': {'before': 'none', 'after': 'exists'}},
     MOCK_CONFIRMY_FILE, OK],

    ['Verify save mock server with cmd line params to empty connections file.',
     {'args': ['save', 'mocktest2'],
      'general': ['--mock-server', MOCK_FILE_PATH,
                  '--timeout', '45',
                  '--use-pull', 'no',
                  '--default-namespace', 'root/blah',
                  '--user', 'john',
                  '--password', 'pw',
                  '--no-verify',
                  '--certfile', 'mycertfile.pem',
                  '--keyfile', 'mykeyfile.pem']},
     {'stdout': "",
      'test': 'lines',
      'file': {'before': 'exists', 'after': 'exists'}},
     None, OK],

    ['Verify connection command shows mock file ',
     ['show', 'mocktest2'],
     {'stdout': [
         "name: mocktest2",
         "default-namespace: root/blah",
         "user: john",
         "password: **",
         "timeout: 45",
         "verify: False",
         "certfile: mycertfile.pem",
         "keyfile: mykeyfile.pem",
         "mock-server:", "simple_mock_model.mof"],
      'test': 'innows',
      'file': {'before': 'exists', 'after': 'exists'}},
     None, OK],

    ['Verify connection command shows svrtest2 ',
     ['show', 'svrtest2'],
     {'stdout': [
         "name: svrtest2",
         "server: http://blah",
         "default-namespace: root/blah", "user: john",
         "password: **",
         "timeout: 45",
         "verify: True",
         "certfile: mycertfile.pem",
         "keyfile: mykeyfile.pem"],
      'test': 'in',
      'file': {'before': 'exists', 'after': 'exists'}},
     None, OK],

    ['Verify connection command delete works',
     ['delete', 'mocktest2'],
     {'stdout': "",
      'test': 'innows',
      'file': {'before': 'exists', 'after': 'exists'}},
     None, OK],

    ['Verify connection command delete works and file empty',
     ['delete', 'svrtest2'],
     {'stderr': "",
      'rc': 0,
      'test': 'innows',
      'file': {'before': 'exists', 'after': 'none'}},
     None, OK],

    #
    #  Test select variations
    #

    ['Verify connection show with just current server defined by --server',
     {'args': ['show'],
      'general': ['--server', 'http://blah']},
     {'stdout': ['name', 'not-saved'],
      'test': 'innows',
      'file': {'before': 'none', 'after': 'none'}},
     None, OK],

    ['Verify connection list with current server from general options, plain',
     {'args': ['list'],
      'general': ['--server', 'http://blah', '--output-format', 'plain']},
     {'stdout': ['WBEM server connections: (#: default, *: current)',
                 '*not-saved http://blah root/cimv2  30 True'],
      'test': 'innows',
      'file': {'before': 'none', 'after': 'none'}},
     None, OK],

    #
    #  Sequence to test
    #

    ['Verify Create new connection names svrtest works.',
     {'general': ['--server', 'http://blah'],
      'args': ['save', 'svrtest']},
     {'stdout': "",
      'test': 'lines',
      'file': {'before': 'none', 'after': 'exists'}},
     None, OK],

    ['Verify connection list with current server from general options, grid',
     {'args': ['list'],
      'general': ['--server', 'http://blah']},
     {'stdout': """
WBEM server connections: (#: default, *: current)
+------------+-------------+-------------+-----------+----------+
| name       | server      | namespace   |   timeout | verify   |
|------------+-------------+-------------+-----------+----------|
| *not-saved | http://blah | root/cimv2  |        30 | True     |
| svrtest    | http://blah | root/cimv2  |        30 | True     |
+------------+-------------+-------------+-----------+----------+
""",
      'test': 'innows',
      'file': {'before': 'exists', 'after': 'exists'}},
     None, OK],

    ['Verify connection select of svrtest',
     {'args': ['select', 'svrtest', '--default'],
      'general': ['--server', 'http://blah']},
     {'stdout': '"svrtest" default and current',
      'test': 'innows',
      'file': {'before': 'exists', 'after': 'exists'}},
     None, OK],

    ['Verify connection select of svrtest as default',
     {'args': ['show'],
      'general': []},
     {'stdout': ['svrtest'],
      'test': 'innows',
      'file': {'before': 'exists', 'after': 'exists'}},
     None, OK],

    ['Verify current connection set with general opts overrides default',
     {'args': ['show'],
      'general': ['--server', 'https://blahblahblah']},
     {'stdout': ['not-saved', 'https://blahblahblah'],
      'test': 'innows',
      'file': {'before': 'exists', 'after': 'exists'}},
     None, OK],

    ['Verify connection delete svrtest',
     {'args': ['delete', 'svrtest'],
      'general': []},
     {'stdout': "Deleted default connection",
      'test': 'innows',
      'file': {'before': 'exists', 'after': 'none'}},
     None, OK],

    #
    #   Test creating a server and saving it from stdin all in single
    #   interactive sequence
    #
    ['Verify Create new connection in interactive mode and delete.',
     {'general': [],
      'stdin': ['--server http://blah --user fred --password fred '
                '--certfile cert1.pem --keyfile keys1.pem',
                'connection save fred',
                'connection show',
                'connection delete fred']},
     {'stdout': ['name', 'not-saved',
                 'server', 'http://blah',
                 'default-namespace', 'root/cimv2',
                 'user',
                 'fred',
                 'password',
                 'timeout', '30',
                 'verify', 'True',
                 'certfile', 'cert1.pem',
                 'keyfile', 'keys1.pem',
                 'Deleted', 'connection', '"fred"'],
      'test': 'innows',
      'file': {'before': 'none', 'after': 'none'}},
     None, OK],

    #
    # The following is a sequence to assure that the interactive creation
    # of a server actually puts the server in the repository
    #
    ['Verify Create new connection in interactive mode and delete.',
     {'general': [],
      'stdin': ['--server http://blah --user fred --password fred '
                '--certfile cert1.pem --keyfile keys1.pem',
                'connection save fred',
                'connection show']},
     {'stdout': ['name', 'not-saved',
                 'server', 'http://blah',
                 'default-namespace', 'root/cimv2',
                 'user',
                 'fred',
                 'password',
                 'timeout', '30',
                 'verify', 'True',
                 'certfile', 'cert1.pem',
                 'keyfile', 'keys1.pem'],
      'test': 'innows',
      'file': {'before': 'none', 'after': 'exists'}},
     None, OK],

    ['Verify connection delete fred',
     {'args': ['delete', 'fred'],
      'general': []},
     {'stdout': 'Deleted connection "fred"',
      'test': 'innows',
      'file': {'before': 'exists', 'after': 'none'}},
     None, OK],

    #
    #   The following is a sequence. Confirm fix to issue #396
    #
    ['Create mock server defintion.',
     {'args': ['save', 'mocktest3'],
      'general': ['--mock-server', MOCK_FILE_PATH]},
     {'stdout': "",
      'test': 'innows',
      'file': {'before': 'none', 'after': 'exists'}},
     None, OK],

    ['Verify sequence select, list, create new and save, list works.',
     {'general': [],
      'stdin': ['connection select mocktest3',
                'connection list',
                '--server http://blah --user fred --password fred '
                '--certfile cert1.pem connection save testconn',
                'connection connection list',
                'connection select testconn',
                'connection delete testconn',
                'connection delete mocktest3']},
     {'stdout': ['Deleted connection "mocktest3"'],
      'test': 'innows',
      'file': {'before': 'exists', 'after': 'None'}},
     None, OK],

]


class TestSubcmdClass(CLITestsBase):
    """
    Test all of the class command variations.
    """
    command_group = 'connection'

    @pytest.mark.parametrize(
        "desc, inputs, exp_response, mock, condition",
        TEST_CASES)
    def test_connection(self, desc, inputs, exp_response, mock, condition,
                        repo_file_path):
        """
        Common test method for those commands and options in the
        connection command group that can be tested.  Note the
        fixture remove_connection_file which is session, autouse so should
        hide any existing connection file at beginning of this test and
        restore it after the test.
        """
        # Where is this file to be located for tests.
        pywbemserversfile = repo_file_path

        def test_file_existence(file_test):
            """
            Local function to execute tests on existence of connections file.
            """
            if file_test == 'exists':
                assert os.path.isfile(pywbemserversfile) and \
                    os.path.getsize(pywbemserversfile) > 0, \
                    "Fail. File  %s should exist" % pywbemserversfile
            elif file_test.lower() == 'none':
                if os.path.isfile(pywbemserversfile):
                    print('FILE THAT SHOULD NOT EXIST')
                    with open(pywbemserversfile, 'r') as fin:
                        print(fin.read())

                assert not os.path.isfile(pywbemserversfile), \
                    "Fail. File %s should not exist" % pywbemserversfile

            else:
                assert False, 'File test option name %s invalid' % file_test

        if not condition:
            pytest.skip("Condition for test case not met")

        if condition == 'pdb':
            import pdb  # pylint: disable=import-outside-toplevel
            pdb.set_trace()

        if 'file' in exp_response:
            if 'before' in exp_response['file']:
                test_file_existence(exp_response['file']['before'])

        self.command_test(desc, self.command_group, inputs, exp_response,
                          mock, condition, verbose=False)

        if 'file' in exp_response:
            if 'after' in exp_response['file']:
                test_file_existence(exp_response['file']['after'])



#!/usr/bin/env python


# (C) Copyright 2017 IBM Corp.
# (C) Copyright 2017 Inova Development Inc.
# All Rights Reserved
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
Tests for functions in smipyping/_common.py
"""
from __future__ import print_function, absolute_import

import os
import unittest
import sys

try:
    from StringIO import StringIO
except ImportError:
    from io import StringIO

from pywbemtools.pywbemcli._common import format_table, fold_string

VERBOSE = False

# TODO ks rewrite this test for pytest.  Note that the capture IO
#      may be different in that they have fixtures such as capsys
#      to capture output that may replace StringIO used below.


class BaseTableTests(unittest.TestCase):
    """Base class for testing table output"""
    @staticmethod
    def create_simple_table(table_format=None, title=True):
        """Create a standard table"""
        headers = ['col1', 'col2', 'col3']
        rows = [['row1col1', 'row1col2', 'row1col3'],
                ['row2col1', 'row2col2', 'row2col3'],
                ['row3 col1', 'row3  col2', 'row3   col3'],
                [0, 999, 9999999],
                [1.1432, 1.2, 0]]
        title_txt = 'test simple table' if title else None
        table = format_table(rows, headers, title=title_txt,
                             table_format=table_format)
        return table

    @staticmethod
    def create_sortable_table(table_format=None, title=True, sort_columns=None):
        """Create a standard table"""
        headers = ['col1', 'col2', 'col3']
        rows = [['row1col1', 'row1col2', 1],
                ['row2col1', 'row2col2', 99],
                ['row3 col1', 'row3  col2', 0],
                ['anotherrow', 'middle', 9999999],
                ['xorerow', 'anuts', 0]]
        title_txt = 'test sortable table' if title else None
        table = format_table(rows, headers, title=title_txt,
                             table_format=table_format,
                             sort_columns=sort_columns)
        return table

    @staticmethod
    def create_folded_table(table_format, title=True):
        """Create a table with folded cells"""
        headers = ['col1', 'col2', 'col3']
        folded = fold_string('this is a folded cell', 10)
        rows = [['row1col1', 'row2col2', folded],
                [folded, 'row2col2', 'row2col3']]
        title_txt = 'test folded table' if title else None
        table = format_table(rows, headers, title=title_txt,
                             table_format=table_format)
        return table

    @staticmethod
    def compare_results(actual, expected):
        """Compare two multiline strings to find differences. Helpful to
           find minor errors in the actual/expected outputs
        """
        if actual == expected:
            return
        actual_lines = actual.split(os.linesep)
        expected_lines = expected.split(os.linesep)
        if len(actual_lines) != len(expected_lines):
            print('Different number of lines actual %s, expected %s' %
                  (len(actual_lines), len(expected_lines)))
        line = 0
        for line_a, line_e in zip(actual_lines, expected_lines):
            if line_a != line_e:
                print('Line %s: Difference\n%s\n%s' % (line, line_a, line_e))
                if len(line_a) != len(line_e):
                    print('Different lengths act %s exp %s' % (len(line_a),
                                                               len(line_e)))
            line += 1


class FormatTableTests(BaseTableTests):
    """Tests on the table_format function in _common.py"""

    def test_table_table_hdr(self):
        """Test a table output_format table with header"""

        captured_output = StringIO()          # Create StringIO object
        sys.stdout = captured_output                   # and redirect stdout.
        table = self.create_simple_table(table_format='table', title=True)
        print(table)
        sys.stdout = sys.__stdout__
        actual = captured_output.getvalue()
        if VERBOSE:
            print(actual)

        expected = ('test simple table\n'
                    '+-----------+------------+-------------+\n'
                    '| col1      | col2       | col3        |\n'
                    '|-----------+------------+-------------|\n'
                    '| row1col1  | row1col2   | row1col3    |\n'
                    '| row2col1  | row2col2   | row2col3    |\n'
                    '| row3 col1 | row3  col2 | row3   col3 |\n'
                    '| 0         | 999        | 9999999     |\n'
                    '| 1.1432    | 1.2        | 0           |\n'
                    '+-----------+------------+-------------+\n')

        self.compare_results(actual, expected)

        self.assertEqual(actual, expected,
                         'Actual:\n%s\nExpected:\n%s\n' % (actual, expected))

    def test_tpsql_table_hdr(self):
        """Test a table output_format table with header"""

        captured_output = StringIO()          # Create StringIO object
        sys.stdout = captured_output                   # and redirect stdout.
        table = self.create_simple_table(table_format='psql', title=True)
        print(table)
        sys.stdout = sys.__stdout__
        actual = captured_output.getvalue()
        if VERBOSE:
            print(actual)

        expected = ('test simple table\n'
                    '+-----------+------------+-------------+\n'
                    '| col1      | col2       | col3        |\n'
                    '|-----------+------------+-------------|\n'
                    '| row1col1  | row1col2   | row1col3    |\n'
                    '| row2col1  | row2col2   | row2col3    |\n'
                    '| row3 col1 | row3  col2 | row3   col3 |\n'
                    '| 0         | 999        | 9999999     |\n'
                    '| 1.1432    | 1.2        | 0           |\n'
                    '+-----------+------------+-------------+\n')

        self.compare_results(actual, expected)

        self.assertEqual(actual, expected,
                         'Actual:\n%s\nExpected:\n%s\n' % (actual, expected))

    def test_html_table_hdr(self):
        """Test a table output_format table with header"""

        captured_output = StringIO()          # Create StringIO object
        sys.stdout = captured_output                   # and redirect stdout.
        table = self.create_simple_table(table_format='html', title=True)
        print(table)
        sys.stdout = sys.__stdout__
        actual = captured_output.getvalue()
        if VERBOSE:
            print(actual)

        expected = (
            '<p>test simple table</p>\n'
            '<table>\n'
            '<thead>\n'
            '<tr><th>col1     </th><th>col2      </th><th>col3       '
            '</th></tr>\n'
            '</thead>\n'
            '<tbody>\n'
            '<tr><td>row1col1 </td><td>row1col2  </td><td>row1col3   '
            '</td></tr>\n'
            '<tr><td>row2col1 </td><td>row2col2  </td><td>row2col3   '
            '</td></tr>\n'
            '<tr><td>row3 col1</td><td>row3  col2</td><td>row3   '
            'col3</td></tr>\n'
            '<tr><td>0        </td><td>999       </td><td>9999999    '
            '</td></tr>\n'
            '<tr><td>1.1432   </td><td>1.2       </td><td>0          '
            '</td></tr>\n'
            '</tbody>\n'
            '</table>\n')

        self.compare_results(actual, expected)

        self.assertEqual(actual, expected,
                         'Actual:\n%s\nExpected:\n%s\n' % (actual, expected))

    def test_table_simple_hdr(self):
        """Test a simple table with header"""

        captured_output = StringIO()          # Create StringIO object
        sys.stdout = captured_output                   # and redirect stdout.
        table = self.create_simple_table(table_format='simple', title=True)
        print(table)
        sys.stdout = sys.__stdout__
        actual = captured_output.getvalue()
        if VERBOSE:
            print(actual)

        expected = ('test simple table\n'
                    'col1       col2        col3\n'
                    '---------  ----------  -----------\n'
                    'row1col1   row1col2    row1col3\n'
                    'row2col1   row2col2    row2col3\n'
                    'row3 col1  row3  col2  row3   col3\n'
                    '0          999         9999999\n'
                    '1.1432     1.2         0\n')

        self.compare_results(actual, expected)

        self.assertEqual(actual, expected,
                         'Actual:\n%s\nExpected:\n%s\n' % (actual, expected))

    def test_table_simple_hdr_sort1(self):
        """Test a simple table with header and the sort option"""

        captured_output = StringIO()          # Create StringIO object
        sys.stdout = captured_output                   # and redirect stdout.
        table = self.create_sortable_table(table_format='simple', title=True,
                                           sort_columns=0)
        print(table)
        sys.stdout = sys.__stdout__
        actual = captured_output.getvalue()
        if VERBOSE:
            print(actual)

        expected = ('test sortable table\n'
                    'col1        col2           col3\n'
                    '----------  ----------  -------\n'
                    'anotherrow  middle      9999999\n'
                    'row1col1    row1col2          1\n'
                    'row2col1    row2col2         99\n'
                    'row3 col1   row3  col2        0\n'
                    'xorerow     anuts             0\n')

        self.compare_results(actual, expected)

        self.assertEqual(actual, expected,
                         'Actual:\n%s\nExpected:\n%s\n' % (actual, expected))

    def test_table_simple_hdr_sort2(self):
        """Test a simple table with header and the sort option"""

        captured_output = StringIO()          # Create StringIO object
        sys.stdout = captured_output                   # and redirect stdout.
        table = self.create_sortable_table(table_format='simple', title=True,
                                           sort_columns=[1, 0])
        print(table)
        sys.stdout = sys.__stdout__
        actual = captured_output.getvalue()
        if VERBOSE:
            print(actual)

        expected = ('test sortable table\n'
                    'col1        col2           col3\n'
                    '----------  ----------  -------\n'
                    'xorerow     anuts             0\n'
                    'anotherrow  middle      9999999\n'
                    'row1col1    row1col2          1\n'
                    'row2col1    row2col2         99\n'
                    'row3 col1   row3  col2        0\n')

        self.compare_results(actual, expected)

        self.assertEqual(actual, expected,
                         'Actual:\n%s\nExpected:\n%s\n' % (actual, expected))

    def test_table_simple_hdr_sort3(self):
        """Test a simple table with header and the sort option"""

        captured_output = StringIO()          # Create StringIO object
        sys.stdout = captured_output                   # and redirect stdout.
        table = self.create_sortable_table(table_format='simple', title=True,
                                           sort_columns=2)
        print(table)
        sys.stdout = sys.__stdout__
        actual = captured_output.getvalue()
        if VERBOSE:
            print(actual)

        expected = ('test sortable table\n'
                    'col1        col2           col3\n'
                    '----------  ----------  -------\n'
                    'row3 col1   row3  col2        0\n'
                    'xorerow     anuts             0\n'
                    'row1col1    row1col2          1\n'
                    'row2col1    row2col2         99\n'
                    'anotherrow  middle      9999999\n')

        self.compare_results(actual, expected)

        self.assertEqual(actual, expected,
                         'Actual:\n%s\nExpected:\n%s\n' % (actual, expected))

    def test_table_simple_no_hdr(self):
        """Test print a simple table no header"""

        captured_output = StringIO()          # Create StringIO object
        sys.stdout = captured_output                   # and redirect stdout.
        table = self.create_simple_table(table_format='simple', title=False)
        print(table)
        sys.stdout = sys.__stdout__
        actual = captured_output.getvalue()
        if VERBOSE:
            print(actual)

        expected = ('col1       col2        col3\n'
                    '---------  ----------  -----------\n'
                    'row1col1   row1col2    row1col3\n'
                    'row2col1   row2col2    row2col3\n'
                    'row3 col1  row3  col2  row3   col3\n'
                    '0          999         9999999\n'
                    '1.1432     1.2         0\n')

        self.compare_results(actual, expected)

        self.assertEqual(actual, expected,
                         'Actual:\n%s\nExpected:\n%s\n' % (actual, expected))

    def test_table_plain_hdr(self):
        """Test a none table borders with header"""

        captured_output = StringIO()          # Create StringIO object
        sys.stdout = captured_output                   # and redirect stdout.

        table = self.create_simple_table(table_format='plain', title=True)
        print(table)

        sys.stdout = sys.__stdout__
        actual = captured_output.getvalue()
        if VERBOSE:
            print(actual)

        expected = ('test simple table\n'
                    'col1       col2        col3\n'
                    'row1col1   row1col2    row1col3\n'
                    'row2col1   row2col2    row2col3\n'
                    'row3 col1  row3  col2  row3   col3\n'
                    '0          999         9999999\n'
                    '1.1432     1.2         0\n')

        self.compare_results(actual, expected)
        self.assertEqual(actual, expected,
                         'Actual:\n%s\nExpected:\n%s\n' % (actual, expected))

    def test_table_grid(self):
        """Test printing a plain table with borders and header"""
        captured_output = StringIO()          # Create StringIO object
        sys.stdout = captured_output                   # and redirect stdout.

        table = self.create_simple_table(table_format='grid', title=True)
        print(table)

        sys.stdout = sys.__stdout__
        actual = captured_output.getvalue()
        if VERBOSE:
            print(actual)

        expected = ('test simple table\n'
                    '+-----------+------------+-------------+\n'
                    '| col1      | col2       | col3        |\n'
                    '+===========+============+=============+\n'
                    '| row1col1  | row1col2   | row1col3    |\n'
                    '+-----------+------------+-------------+\n'
                    '| row2col1  | row2col2   | row2col3    |\n'
                    '+-----------+------------+-------------+\n'
                    '| row3 col1 | row3  col2 | row3   col3 |\n'
                    '+-----------+------------+-------------+\n'
                    '| 0         | 999        | 9999999     |\n'
                    '+-----------+------------+-------------+\n'
                    '| 1.1432    | 1.2        | 0           |\n'
                    '+-----------+------------+-------------+\n')

        self.compare_results(actual, expected)
        self.assertEqual(actual, expected,
                         'Actual:\n%s\nExpected:\n%s\n' % (actual, expected))

    def test_table_rst_hdr(self):
        """Test a none table borders with header"""

        captured_output = StringIO()          # Create StringIO object
        sys.stdout = captured_output                   # and redirect stdout.

        table = self.create_simple_table(table_format='rst', title=True)
        print(table)

        sys.stdout = sys.__stdout__
        actual = captured_output.getvalue()
        if VERBOSE:
            print(actual)

        expected = ('test simple table\n'
                    '=========  ==========  ===========\n'
                    'col1       col2        col3\n'
                    '=========  ==========  ===========\n'
                    'row1col1   row1col2    row1col3\n'
                    'row2col1   row2col2    row2col3\n'
                    'row3 col1  row3  col2  row3   col3\n'
                    '0          999         9999999\n'
                    '1.1432     1.2         0\n'
                    '=========  ==========  ===========\n')

        self.compare_results(actual, expected)
        self.assertEqual(actual, expected,
                         'Actual:\n%s\nExpected:\n%s\n' % (actual, expected))

    def test_folded_cell_plain(self):
        """Test building a folded cell table plain with header"""
        actual = self.create_folded_table(table_format='plain', title=True)

        if VERBOSE:
            print(actual)

        expected = ('test folded table\n'
                    'col1       col2      col3\n'
                    'row1col1   row2col2  this is a\n'
                    '                     folded\n'
                    '                     cell\n'
                    'this is a  row2col2  row2col3\n'
                    'folded\n'
                    'cell')

        self.compare_results(actual, expected)
        self.assertEqual(actual, expected,
                         'Actual:\n%s\nExpected:\n%s\n' % (actual, expected))

    def test_folded_cell_simple(self):
        """Test a folded cell table simplewith header"""
        captured_output = StringIO()          # Create StringIO object
        sys.stdout = captured_output                   # and redirect stdout.

        table = self.create_folded_table(table_format='simple', title=True)
        print(table)

        sys.stdout = sys.__stdout__
        actual = captured_output.getvalue()
        if VERBOSE:
            print(actual)

        expected = ('test folded table\n'
                    'col1       col2      col3\n'
                    '---------  --------  ---------\n'
                    'row1col1   row2col2  this is a\n'
                    '                     folded\n'
                    '                     cell\n'
                    'this is a  row2col2  row2col3\n'
                    'folded\n'
                    'cell\n')

        self.compare_results(actual, expected)
        self.assertEqual(actual, expected,
                         'Actual:\n%s\nExpected:\n%s\n' % (actual, expected))

    def test_folded_cell_grid(self):
        """Test a folded cell table with header"""
        actual = self.create_folded_table(table_format='grid', title=True)

        if VERBOSE:
            print(actual)

        expected = ('test folded table\n'
                    '+-----------+----------+-----------+\n'
                    '| col1      | col2     | col3      |\n'
                    '+===========+==========+===========+\n'
                    '| row1col1  | row2col2 | this is a |\n'
                    '|           |          | folded    |\n'
                    '|           |          | cell      |\n'
                    '+-----------+----------+-----------+\n'
                    '| this is a | row2col2 | row2col3  |\n'
                    '| folded    |          |           |\n'
                    '| cell      |          |           |\n'
                    '+-----------+----------+-----------+')

        self.compare_results(actual, expected)
        self.assertEqual(actual, expected,
                         'Actual:\n%s\nExpected:\n%s\n' % (actual, expected))


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python

# (C) Copyright 2017 IBM Corp.
# (C) Copyright 2017 Inova Development Inc.
# All Rights Reserved
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
    Execute and test the validity of the help output from pywbemcli
"""

from __future__ import print_function, absolute_import

import unittest
import re
from subprocess import Popen, PIPE
import six


class ClientTest(unittest.TestCase):
    """Top level container. Performs any setup and teardown"""

    def setUp(self):
        """Setup the test
        """
        self.host = 'http://localhost'
        self.verbose = False


class TestsContainer(ClientTest):
    """Container class for all tests"""

    def execute_cmd(self, cmd_str):  # pylint: disable=no-self-use
        """Execute the command defined by cmd_str and return results."""
        if self.verbose:
            print('cmd %s' % cmd_str)
        # Disable python warnings for pywbemcli call.See issue #42
        command = 'export PYTHONWARNINGS="" && %s' % cmd_str
        proc = Popen(command, shell=True, stdout=PIPE, stderr=PIPE)
        std_out, std_err = proc.communicate()
        exitcode = proc.returncode
        if six.PY3:
            std_out = std_out.decode()
            std_err = std_err.decode()
        if self.verbose:
            print('rtn %s\n%s\n%s' % (std_out, std_err, exitcode))

        # return tuple of exitcode, stdout, stderr
        return exitcode, std_out, std_err

    def assert_not_found(self, regex, test_str):
        """ Test of find regex on multiline string.
        If regex is a list each entry is tested.
        """
        if isinstance(regex, list):
            for i in regex:
                self.assert_not_found(i, test_str)
        else:
            match = re.search(regex, test_str)
            if match:
                self.fail('Found in error search regex %s, str %s' %
                          (regex, test_str))

    def assert_found(self, regex, test_str):
        """ Test of find regex on multiline string.
        If regex is a list each entry is tested.
        """
        if isinstance(regex, list):
            for i in regex:
                self.assert_found(i, test_str)
        else:
            match = re.search(regex, test_str)
            if match is None:
                self.fail('Failed search regex %s, str %s' % (regex, test_str))

    def assertRegexp(self, regex, test_str):
        """
        This function eliminates the issue between the unittest assertRegex
        and assertRegexpMatches functions between unittiest in python 2 and 3
        """
        if six.PY3:
            # pylint: disable=no-member
            return self.assertRegex(test_str, regex)

        # pylint: disable=no-member, deprecated-method
        return self.assertRegexpMatches(test_str,
                                        regex)  # pylint: disable=no-member


class ClassTests(TestsContainer):
    """Test operations in the class group"""

    def class_cmd(self, params):
        """Adds the cmd name prefix and executes"""
        cmd = 'pywbemcli -s %s class %s' % (self.host, params)
        exitcode, std_out_str, std_err_str = self.execute_cmd(cmd)
        return exitcode, std_out_str, std_err_str

    def test_get_simple(self):
        """Test a get of CIM_ManagedElement"""
        exitcode, out, err = self.class_cmd('get CIM_ManagedElement')

        self.assertEqual(exitcode, 0)
        self.assertEqual(err, '', 'Expect no std_err. Found %s' % err)
        self.assert_found('CIM_ManagedElement', out)

    def test__get_localonly(self):
        """Test class get --local-only"""
        exitcode, out, err = self.class_cmd('get CIM_ManagedElement -l')

        self.assertEqual(exitcode, 0)
        self.assertEqual(err, "")
        self.assert_found('CIM_ManagedElement', out)

        exitcode, out, err = self.class_cmd(
            'get CIM_ManagedElement --local-only')

        self.assertEqual(exitcode, 0)
        self.assert_found('CIM_ManagedElement', out)

    def test_get_no_includequalifiers(self):
        """ """
        exitcode, out, err = self.class_cmd(
            'get CIM_ManagedElement --no-qualifiers')

        self.assertEqual(exitcode, 0)
        self.assert_found('CIM_ManagedElement', out)

    def test_propertylist(self):
        """Test property list on the get"""
        exitcode, out, err = self.class_cmd(
            'get CIM_ManagedElement -p InstanceID')
        self.assertEqual(exitcode, 0)
        self.assert_found(['class CIM_ManagedElement', 'InstanceID'], out)

        exitcode, out, err = self.class_cmd(
            'get CIM_ManagedElement -p InstanceID -p Caption')
        self.assertEqual(exitcode, 0)
        self.assert_found('class CIM_ManagedElement', out)
        self.assert_found('InstanceID', out)
        self.assert_found('Caption', out)

        exitcode, out, err = self.class_cmd(
            'get CIM_ManagedElement -p ""')
        self.assertEqual(exitcode, 0)
        self.assert_found('class CIM_ManagedElement', out)
        self.assert_not_found(['InstanceID', 'Caption'], out)

    def test_simple_invoke(self):
        """Execute simple invoke method defined in pegasus"""
        exitcode, out, err = self.class_cmd(
            'invokemethod Test_IndicationProviderClass '
            'SendTestIndicationsCount -p indicationSendCount=0 '
            ' -n test/TestProvider')
        self.assertEqual(exitcode, 0)


# TODO finish this based on the test_ops in the tools directory

# cmd "class get CIM_ManagedElement -c"
# cmd "class get CIM_ManagedElement --include-classorigin"
# cmd "class get CIM_ManagedElement --namespace root/PG_Interop"
# cmd "class get CIM_ManagedElement - root/PG_Interop"

# TODO create tests for qualifier, server

class InstanceTests(TestsContainer):
    """Test operations in the instance group"""

    def instance_cmd(self, params):
        """Adds the instance cmd name prefix and executes"""
        cmd = 'pywbemcli -s %s instance %s' % (self.host, params)
        exitcode, std_out_str, std_err_str = self.execute_cmd(cmd)
        return exitcode, std_out_str, std_err_str

    def test_enumerate_simple(self):
        """ """
        exitcode, out, err = self.instance_cmd('enumerate PyWBEM_Person')

        self.assertEqual(exitcode, 0)
        self.assert_found('instance of PyWBEM_Person', out)

    def test_enumerate_proplist(self):
        """ """
        exitcode, out, err = self.instance_cmd('enumerate PyWBEM_Person '
                                               '-p Name')

        self.assertEqual(exitcode, 0)
        self.assert_found(['instance of PyWBEM_Person', 'Name'], out)
        self.assert_not_found('CreationClassName', out)

    def test_get_simple(self):
        """Execute simple get of known instance """
        exitcode, out, err = self.instance_cmd(
            'get PyWBEM_Person.CreationClassname=PyWBEM_Person,Name=Bob')

        self.assertEqual(exitcode, 0)
        self.assert_found('PyWBEM_Person', out)

    def test_create_simple(self):
        """
        Test create a simple instance. To be complete this must both
        create and delete the instance since tests are not ordered and each
        test should leave the repository in the same state in which it
        was before the test.
        """
        exitcode, out, err = self.instance_cmd(
            'create PyWBEM_Person --property name=Fred '
            '--property CreationClassname=PyWBEM_Person')
        self.assertEqual(exitcode, 0)

        exitcode, out, err = self.instance_cmd(
            'delete PyWBEM_Person.Name=Fred,CreationClassName=PyWBEM_Person')
        self.assertEqual(exitcode, 0)
        self.assert_found(['Deleted', 'Fred'], out)

    def test_create_array_prop(self):
        """Create an instance of an array property"""

        exitcode, out, err = self.instance_cmd(
            'create pywbem_alltypes --property InstanceId=ArrayBool '
            '--property arrayBool=True,False')

        self.assertEqual(exitcode, 0, "Failed create test")
        exitcode, out, err = self.instance_cmd(
            'get pywbem_alltypes.InstanceId=ArrayBool')

        self.assert_found(["instance of PyWBEM_AllTypes", 'ArrayBool',
                           "{True, False}"], out)

        exitcode, out, err = self.instance_cmd(
            'delete PyWBEM_AllTypes.InstanceId=ArrayBool')
        self.assertEqual(exitcode, 0)
        self.assert_found(['Deleted', 'ArrayBool'], out)

    def test_create_alltypes(self):
        """
        Create an instance of a class with all types
        """

        exitcode, out, err = self.instance_cmd(
            'create PyWBEM_AllTypes --property InstanceId=ScalarTest1 '
            '--property scalBool=True '
            '--property scalUint8=8 '
            '--property scalSint8=-8 '
            '--property scalUint32=9999 '
            '--property scalSint32=-9999 '
            '--property scalUint64=12345678 '
            '--property scalSint64=-12345678 '
            '--property scalReal32=5678.32 '
            '--property scalReal64=345876.3 '
            '--property scalDateTime="19991224120000.000000+360" '
            '--property scalString="A string value" ')
        self.assertEqual(exitcode, 0, 'Expected good response. Rcvd '
                         ' code %s err %s' % (exitcode, err))
        self.assertEqual(exitcode, 0, 'Create instance of Pywbem_AllTypes '
                         'failed. exitcode %s, err %s' % (exitcode, err))

        exitcode, out, err = self.instance_cmd(
            'delete PyWBEM_AllTypes.InstanceId=ScalarTest1')
        self.assertEqual(exitcode, 0)
        self.assert_found(['Deleted', 'ScalarTest1'], out)

    def test_property_notexist(self):
        """
        Validate the error when property does not exist in class
        """
        exitcode, out, err = self.instance_cmd(
            'create pywbem_alltypes --property InstanceId=ArrayBool '
            '--property BlahBool=True,False')
        print('err %s' % err)
        self.assertEqual(exitcode, 1)

    def test_references(self):
        exitcode, out, err = self.instance_cmd(
            'references PyWBEM_Person.CreationClassname=PyWBEM_Person,'
            'Name=Bob')

        self.assertEqual(exitcode, 0)
        self.assert_found('instance of PyWBEM_MemberOfPersonCollection', out)

    def test_reference_paths(self):
        exitcode, out, err = self.instance_cmd(
            'references PyWBEM_Person.CreationClassname=PyWBEM_Person,'
            'Name=Bob -o')

        self.assertEqual(exitcode, 0)
        self.assert_found(':PyWBEM_MemberOfPersonCollection.Member', out)

    def test_associators(self):
        exitcode, out, err = self.instance_cmd(
            'associators PyWBEM_Person.CreationClassname=PyWBEM_Person,'
            'Name=Bob')

        self.assertEqual(exitcode, 0)
        self.assert_found('instance of PyWBEM_PersonCollection', out)

    def test_associator_paths(self):
        exitcode, out, err = self.instance_cmd(
            'associators PyWBEM_Person.CreationClassname=PyWBEM_Person,'
            'Name=Bob -o')

        self.assertEqual(exitcode, 0)
        self.assert_found(':PyWBEM_PersonCollection.InstanceID', out)


if __name__ == '__main__':
    unittest.main()



# (C) Copyright 2017 IBM Corp.
# (C) Copyright 2017 Inova Development Inc.
# All Rights Reserved
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
Click Command definition for the qualifer command group  which includes
cmds for get and enumerate for CIM qualifier types.
"""

from __future__ import absolute_import

import six

from asciitree import LeftAligned
import click
from pywbem.cim_obj import NocaseDict


def build_tree(class_subclass_dict, top_class):
    """
    Build a dictionary structure based on the class/subclass relationships
    in the class_subclass tree dictionary provided.

    Returns the dictionary structure in a form suitable for ascii tree
    display
    """
    def _tree_node(class_subclass_dict, cn):
        """
        Build dictionary of the class/subclass relationships for class cn
        in dictionary of class_subclass names.

        Returns dictionary of dictionaries in form suitable for asciitree
        """
        node_dict = NocaseDict()
        # If there is no subclass, the class will not exist in this dictionary
        if cn in class_subclass_dict:
            cn_list = class_subclass_dict[cn]
            # This should not be necessary if end nodes are not in the dict.
            if cn_list:
                for key in cn_list:
                    node_dict[key] = _tree_node(class_subclass_dict, key)
            else:
                node_dict = NocaseDict()
        else:
            return {}

        return node_dict

    rtn_dict = {}
    # _tree_node generates dictionary node for elements in class-subclass
    # dictionary and returns complete node structure
    rtn_dict[top_class] = _tree_node(class_subclass_dict, top_class)
    return rtn_dict


def display_class_tree(classes, top_class=None):
    """
    Display the list of classes as a left justified tree  in ascii to the
    click.echo output

    Parameters:
        classes (list of :class:`~pywbem.CIMClass`)

        top_class (:term: `string`)
            The top level class to display or None if the display is
            from root.
    """

    # build dictionary of classname : superclassname
    cn_supercn = {cl.classname: cl.superclass for cl in classes}

    # if top_class is none, create artifical root
    if top_class is None:
        for cn in cn_supercn:
            if not cn_supercn[cn]:
                cn_supercn[cn] = 'root'
        top_class = 'root'

    # Hack to build the class to subclass dictionary from the
    # superclass to class dictionary
    cn_subcn = NocaseDict()
    # pylint: disable=bad-continuation, expression-not-assigned
    [cn_subcn.setdefault(v, []).append(k) for (k, v) in
            six.iteritems(cn_supercn)]  # noqa: F841
    tree = build_tree(cn_subcn, top_class)

    tr = LeftAligned()
    click.echo(tr(tree))



#!/usr/bin/env python

# (C) Copyright 2017 IBM Corp.
# (C) Copyright 2017 Inova Development Inc.
# All Rights Reserved
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Tests for _contest_obj functions and classes
"""

from __future__ import absolute_import, print_function

import unittest

from pywbemtools.pywbemcli._context_obj import ContextObj
from pywbemtools.pywbemcli._pywbem_server import PywbemServer


class ContextObjTests(unittest.TestCase):
    """Test the click context object class"""

    def simple_test(self):
        """Test create context object"""
        server = 'http://localhost'
        ns = 'root/cimv2'
        user = 'Fred'
        pw = 'blah'
        svr = PywbemServer(server, ns, user=user, password=pw)

        def password_prompt():
            pass
        output_format = 'mof'
        verbose = True
        ctx = None

        ctxobj = ContextObj(ctx, svr, password_prompt, output_format,
                            verbose)
        self.assertEqual(ctxobj.pywbem_server, svr)


if __name__ == '__main__':
    unittest.main()
